#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 08:10:11 PM CST

''' album and item '''

import os
import sys
import time
import urllib
import urlparse

from system import NASError
from system import _all, _any
from system import getrealpath
from system import check_table
from system import sqlite, HOMEDB
from system import getlist, getunicode
from system import pcdata
from cStringIO import StringIO
from login import updatetime

from nas.users import _getusers

import Image
import magic
mime = magic.open(magic.MAGIC_MIME)
if mime != None:
    mime.load()

AUDIO_URL = 'Applications/MyNAS/icons/audio_thumbnail.jpg'
BAD_URL = 'Applications/MyNAS/icons/bad_thumbnail.jpg'

TZ = '%Y-%m-%dT%H:%M:%SZ'
RFC1123 = '%a, %d %b %Y %H:%M:%S GMT'
DIR_MIMETYPE = 'httpd/unix-directory'
DTNS = 'xmlns:dt="urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/"'

MRSS_NS = 'xmlns:media="http://search.yahoo.com/mrss/"'
ATOM_NS = 'xmlns:atom="http://www.w3.org/2005/Atom"'
VIDEOS = ['application/ogg',
          'application/x-pn-realmedia',
          'application/x-shockwave-flash']

PADDING = list('abcdefghijklmnopqrs')

def _getcoverurl(cover, session):
    addr, sep, param = cover.partition('?')
    coverurl = '%s?' % urllib.quote(addr)
    if 'thumbnail' in param:
        coverurl += '&thumbnail'
    elif 'webview' in param:
        coverurl += '&webview'
    if session:
        coverurl += '&session=%s' % session
    return pcdata(coverurl)

def getalbumowner(login=None, **kw):
    if login is None:
        raise NASError('Bad Request')

    login = getunicode(login)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT id, name, cover, comment, creation, modification
                    FROM album WHERE owner = ? ORDER BY name ASC
                ''', (login,))

    body = StringIO()
    body.write('<?xml version="1.0" encoding="utf-8"?>')
    body.write('<D:multistatus xmlns:D="DAV:" xmlns:R="works" %s>' % DTNS)

    for album in cur.fetchall():
        (_id, name, cover, comment, creation, modification) = album
        cur.execute(''' SELECT count() FROM item WHERE album = ?
                    ''', (_id,))
        count = str(cur.fetchone()[0])

        cur.execute(''' SELECT sharer FROM share WHERE album = ?
                    ''', (_id,))
        shares = cur.fetchall()

        body.write('<D:response>')

        body.write('<D:propstat>')
        body.write('<D:prop>')

        body.write('<D:displayname>%s</D:displayname>' % pcdata(name))
        body.write('<D:getcontenttype>%s</D:getcontenttype>' % DIR_MIMETYPE)
        body.write('<D:getcontentlength>%s</D:getcontentlength>' % count)

        body.write('<D:creationdate dt:dt="dateTime.tz">')
        body.write(time.strftime(TZ, time.gmtime(creation)))
        body.write('</D:creationdate>')

        body.write('<D:getlastmodified dt:dt="dateTime.rfc1123">')
        body.write(time.strftime(RFC1123, time.gmtime(modification)))
        body.write('</D:getlastmodified>')

        for share in shares:
            sharer = share[0]
            body.write('<R:sharer>%s</R:sharer>' % pcdata(sharer))

        if cover:
            if isinstance(cover, unicode):
                cover = cover.encode('utf8')
            if getrealpath(cover, login, False):
                body.write('<R:cover>%s</R:cover>' % _getcoverurl(cover, kw.get('session')))

        body.write('</D:prop>')
        body.write('</D:propstat>')

        if comment:
            body.write('<D:responsedescription>')
            body.write('%s' % pcdata(comment))
            body.write('</D:responsedescription>')

        body.write('</D:response>')

    body.write('</D:multistatus>')

    con.close()

    return body

# noauth
def getalbumsharer(session=None, owner=None, share=None, **kw):
    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    if session is not None:
        login = updatetime(session)
    else:
        login = None
    if login is not None and share is not None:
        login = getunicode(login)
        cur.execute(''' SELECT DISTINCT album.id, album.owner, album.name,
                            album.cover, album.comment,
                            album.creation, album.modification
                        FROM album, share
                        WHERE (share.sharer = ? OR share.sharer = ?)
                            AND album.id = share.album
                            AND album.owner != ?
                        ORDER BY album.owner ASC, album.name ASC
                    ''', (login, u'*', login))
    elif login is not None:
        login = getunicode(login)
        cur.execute(''' SELECT id, owner, name, cover, comment,
                            creation, modification
                        FROM album WHERE owner = ?
                        ORDER BY name ASC
                    ''', (login,))
    elif owner is not None:
        owner = getunicode(owner)
        cur.execute(''' SELECT DISTINCT album.id, album.owner, album.name,
                            album.cover, album.comment,
                            album.creation, album.modification
                        FROM album, share
                        WHERE album.owner = ? AND share.sharer = ?
                            AND album.id = share.album
                        ORDER BY album.owner ASC, album.name ASC
                    ''', (owner, u'*'))
    else:
        cur.execute(''' SELECT DISTINCT album.id, album.owner, album.name,
                            album.cover, album.comment,
                            album.creation, album.modification
                        FROM album, share
                        WHERE share.sharer = ? AND album.id = share.album
                        ORDER BY album.owner ASC, album.name ASC
                        ''', (u'*',))

    body = StringIO()
    body.write('<?xml version="1.0" encoding="utf-8"?>')
    body.write('<D:multistatus xmlns:D="DAV:" xmlns:R="works" %s>' % DTNS)

    users = _getusers()
    for album in cur.fetchall():
        (albumid, owner, name, cover, comment, creation, modification) = album
        if owner.encode('utf8') not in users:
            continue
        cur.execute(''' SELECT count() FROM item WHERE album = ?
                    ''', (albumid,))
        count = '%s' % cur.fetchone()[0]

        body.write('<D:response>')

        body.write('<D:propstat>')
        body.write('<D:prop>')


        body.write('<D:displayname>%s</D:displayname>' % pcdata(name))
        body.write('<D:getcontenttype>%s</D:getcontenttype>' % DIR_MIMETYPE)
        body.write('<D:getcontentlength>%s</D:getcontentlength>' % count)

        body.write('<D:creationdate dt:dt="dateTime.tz">')
        body.write(time.strftime(TZ, time.gmtime(creation)))
        body.write('</D:creationdate>')

        body.write('<D:getlastmodified dt:dt="dateTime.rfc1123">')
        body.write(time.strftime(RFC1123, time.gmtime(modification)))
        body.write('</D:getlastmodified>')

        body.write('<R:author>%s</R:author>' % pcdata(owner))

        if cover:
            if isinstance(cover, unicode):
                cover = cover.encode('utf8')
            if getrealpath(cover, owner.encode('utf8'), False):
                body.write('<R:cover>%s</R:cover>' % _getcoverurl(cover, session))

        body.write('</D:prop>')
        body.write('</D:propstat>')

        if comment:
            body.write('<D:responsedescription>')
            body.write(pcdata(comment))
            body.write('</D:responsedescription>')

        body.write('</D:response>')

    body.write('</D:multistatus>')
    con.close()

    return body

def addalbum(login=None, name=None, comment='', cover='', sharer=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    comment = getunicode(comment)
    cover = getunicode(cover)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    try:
        date = time.time()
        cur.execute(''' INSERT INTO album (owner, name, comment, cover,
                            creation, modification)
                        VALUES (?, ?, ?, ?, ?, ?)
                    ''', (login, name, comment, cover, date, date))
    except sqlite.IntegrityError:
        con.close()
        raise NASError('Album Exists', name.encode('utf8'))

    # get album
    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    album = result[0]

    con.commit()
    con.close()

    if sharer is not None:
        shares = getlist(sharer)
        _modifyshare(album, shares)

    return {'album': 'add album'}

def editalbum(login=None, name=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    album = result[0]

    items = ['comment', 'cover', 'newname']
    for item in items:
        if kw.get(item) is not None:
            value = getunicode(kw.get(item))
            if item == 'newname':
                item = 'name'
            try:
                cur.execute(''' UPDATE album SET %s = ?, modification = ?
                                WHERE id = ?
                            ''' % item, (value, time.time(), album))
            except sqlite.IntegrityError:
                con.commit()
                con.close()
                raise NASError('Album Exists', value.encode('utf8'))

    con.commit()
    con.close()

    if kw.get('sharer') is not None:
        shares = getlist(kw.get('sharer'))
        _modifyshare(album, shares)

    return {'album': 'edit album'}

def delalbum(login=None, name=None, **kw):
    if not _all(login, name):
        raise NASError('Bad Request')

    login = getunicode(login)
    names = getlist(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    for name in names:
        cur.execute(''' SELECT id FROM album
                        WHERE owner = ? AND name = ?
                    ''', (login, name))
        album = cur.fetchone()
        if not album:
            continue
        cur.execute(''' DELETE FROM album WHERE id = ? ''', album)
        cur.execute(''' DELETE FROM item WHERE album = ? ''', album)
        cur.execute(''' DELETE FROM share WHERE album = ? ''', album)

    con.commit()
    con.close()

    return {'album': 'delete album'}

# noauth
def getitems(owner=None, name=None, login=None, url=None, **kw):

    if not _any(owner, name, login):
        if url is None:
            raise NASError('Bad Request')

        # /name_owner_session.rss
        paramlist = url[1:-4].split('_')
        if len(paramlist[-1]) == 1:
            raise NASError('Bad Request')

        if len(paramlist[-1]) == 40:
            login = None
            try:
                login = updatetime(paramlist[-1])
            except:
                session = None
                owner = paramlist[-1]
                name  = '_'.join(paramlist[:-1])
            else:
                session = paramlist[-1]
                owner = paramlist[-2]
                name  = '_'.join(paramlist[:-2])
        else:
            session = None
            owner = paramlist[-1]
            name  = '_'.join(paramlist[:-1])

    print 'owner=%s, name=%s, login=%s' % (owner, name, login)
    request = '?'
    if session is not None:
        request += 'session=%s' % session

    owner = getunicode(owner)
    name = getunicode(name)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    if login is None:
        cur.execute(''' SELECT album.id FROM album, share
                        WHERE album.owner = ? AND album.name = ? AND
                            album.id = share.album AND
                            share.sharer = ?
                    ''', (owner, name, u'*'))
    else:
        login = getunicode(login)
        if owner == login:
            cur.execute(''' SELECT id FROM album
                            WHERE owner = ? AND name = ?
                        ''', (owner, name))
        else:
            cur.execute(''' SELECT album.id FROM album, share
                            WHERE album.owner = ? AND album.name = ? AND
                                album.id = share.album AND
                                (share.sharer = ? OR share.sharer = ?)
                        ''', (owner, name, login, u'*'))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))

    album = result[0]
    cur.execute(''' SELECT id, name, path, description
                    FROM item WHERE album = ?
                ''', (album,))

    items = cur.fetchall()

    body = StringIO()
    body.write('<?xml version="1.0" encoding="utf-8"?>')
    body.write('<rss version="2.0" %s %s>' % (MRSS_NS, ATOM_NS))

    body.write('<channel>')
    for (itemid, name, path, description) in items:
        name = name.encode('utf8')
        oldpath = path.encode('utf8')

        path, unused, query = oldpath.partition('?')
        result = urlparse.parse_qs(query, True).get('login')
        if result:
            login = result[0]
        else:
            login = owner

        realpath = getrealpath(path, login, False)
        if not realpath:
            cur.execute(''' DELETE FROM item WHERE album = ? AND path = ?
                        ''', (album, oldpath.decode('utf8', 'ignore')))
            continue

        mimetype = mime.file(realpath).split(';')[0]
        # application/ogg
        # application/x-pn-realmedia
        # application/x-shockwave-flash
        if mimetype[:5] == 'video' or mimetype in VIDEOS:
            filetype = 'video'
        elif mimetype[:5] == 'audio':
            filetype = 'audio'
        elif mimetype[:5] == 'image':
            filetype = 'image'
        else:
            filetype = 'none'
            # cur.execute(''' DELETE FROM item WHERE album = ? AND path = ?
            #             ''', (album, path.decode('utf8', 'ignore')))
            # continue

        parent, unused, child = path.rpartition(os.sep)

        realparent, unused, child = realpath.rpartition(os.sep)
        realthumbnail = '%s/%s/%s.jpg' % (realparent, '.thumbnail', child)
        realwebview = '%s/%s/%s.jpg' % (realparent, '.webview', child)

        try:
            img = Image.open(realthumbnail)
            t_size = img.size
        except IOError:
            t_size = (0, 0)

        try:
            img = Image.open(realwebview)
            w_size = img.size
        except IOError:
            w_size = (0, 0)

        body.write('<item>')
        body.write('<title>%s</title>' % pcdata(name))
        if description:
            body.write('<description>%s</description>' % pcdata(description))
        padding = '%s%d' % (''.join(PADDING[:-len(str(itemid))]), itemid)
        itemname = padding.encode('base64').strip()
        path_url = 'dav/album/%s%s%s' % (itemname, os.path.splitext(realpath)[1], request)
        body.write('<link>%s</link>' % pcdata(path_url))

        media_thumb = '<media:thumbnail type="image/jpeg"'
        media_thumb += ' width="%s" height="%s" url="%s"/>'
        if t_size[0] and t_size[1]:
            thumb_url = '%s&thumbnail' % path_url
            body.write(media_thumb % (t_size[0], t_size[1], pcdata(thumb_url)))
        elif filetype == 'audio':
            body.write(media_thumb % (128, 96, AUDIO_URL))
        elif filetype == 'none':
            body.write(media_thumb % (128, 96, BAD_URL))

        media_content = '<media:content url="%s" type="%s"%s/>'
        if filetype in ['audio', 'video']:
            body.write(media_content % (path_url, mimetype, ''))
        elif w_size[0] and w_size[1]:
            content_url = '%s&webview' % path_url
            extra = ' width="%s" height="%s"' % (w_size[0], w_size[1])
            body.write(media_content % (pcdata(content_url), mimetype, extra))
        else:
            body.write(media_content % (pcdata(path_url), mimetype, ''))

        statinfo = os.stat(realpath)
        datestring = time.strftime('%y-%m-%d', time.gmtime(statinfo.st_ctime))
        body.write('<date>%s</date>' % datestring)
        body.write('<size>%s</size>' % statinfo.st_size)
        body.write('</item>')

    body.write('</channel>')
    body.write('</rss>')

    con.commit()
    con.close()
    return body

def additem(login=None, name=None, item=None, path=None, **kw):
    if not _all(login, name, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    paths = getpaths(path, login)

    if item is not None:
        items = getlist(item)
    else:
        items = [os.path.split(x)[1] for x in paths]

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    album = result[0]

    for (item, path) in zip(items, paths):
        if not path:
            continue
        try:
            cur.execute(''' INSERT INTO item (album, name, path, description)
                            VALUES (?, ?, ?, ?)
                        ''', (album, item, path, u''))
        except sqlite.IntegrityError:
            continue
    con.commit()
    con.close()

    return {'album': 'add items'}

def delitem(login=None, name=None, path=None, **kw):
    if not _all(login, name, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    paths = getpaths(path, login)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    album = result[0]

    for path in paths:
        cur.execute(''' DELETE FROM item WHERE album = ? and path = ?
                    ''', (album, path))

    con.commit()
    con.close()

    return {'album': 'delete items'}

def moveitem(login=None, name=None, newname=None, path=None, **kw):
    if not _all(login, name, newname, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    newname = getunicode(newname)
    paths = getpaths(path, login)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    # get album
    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    album = result[0]

    # get new album
    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, newname))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    newalbum = result[0]

    for path in paths:
        try:
            cur.execute(''' UPDATE item SET album = ?
                            WHERE album = ? and path = ?
                        ''', (newalbum, album, path))
        except sqlite.IntegrityError:
            continue

    con.commit()
    con.close()

    return {'album': 'move items'}

def edititem(login=None, name=None, path=None, item=None,
        description=None, **kw):
    if not _all(login, name, path):
        raise NASError('Bad Request')

    login = getunicode(login)
    name = getunicode(name)
    paths = getpaths(path, login)
    items = getlist(item)
    descriptions = getlist(description)

    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    # get album
    cur.execute(''' SELECT id FROM album
                    WHERE owner = ? AND name = ?
                ''', (login, name))
    result = cur.fetchone()
    if not result:
        con.close()
        raise NASError('Album Not Exists', name.encode('utf8'))
    album = result[0]

    print 'items=%s, paths=%s' % (repr(items), repr(paths))
    for item, path in zip(items, paths):
        if item is None:
            continue
        cur.execute(''' UPDATE item SET name = ?
                        WHERE album = ? AND path = ?
                    ''', (item, album, path))

    for path, description in zip(paths, descriptions):
        cur.execute(''' UPDATE item SET description = ?
                        WHERE album = ? AND path = ?
                    ''', (description, album, path))

    con.commit()
    con.close()

    return {'album': 'edit items'}

def _modifyshare(album, newshares):

    # newshares = [sharer, ..., ]
    if not isinstance(newshares, list):
        con = sqlite.connect(HOMEDB)
        cur = con.cursor()
        cur.execute(''' DELETE FROM share WHERE album = ?
                    ''', (album,))
        con.commit()
        con.close()
        return True

    con = sqlite.connect(HOMEDB)
    cur = con.cursor()

    if u'*' in newshares:
        cur.execute(''' DELETE FROM share WHERE album = ?
                    ''', (album,))
        cur.execute(''' INSERT INTO share (album, sharer) VALUES (?, ?)
                    ''', (album, u'*'))
        con.commit()
        con.close()
        return True

    cur.execute(''' SELECT sharer FROM share WHERE album = ?
                ''', (album,))
    oldshares = cur.fetchall()

    for sharer in oldshares:
        if sharer[0] in newshares:
            oldshares.remove(sharer)
            newshares.remove(sharer[0])

    for sharer in oldshares:
        cur.execute(''' DELETE FROM share WHERE album = ? AND sharer = ?
                    ''', (album, sharer[0]))

    for sharer in newshares:
        if sharer is None:
            continue
        cur.execute(''' INSERT INTO share (album, sharer) VALUES (?, ?)
                    ''', (album, sharer))

    con.commit()
    con.close()
    return True

def getitemowner(itemid, login=None):
    check_table()
    con = sqlite.connect(HOMEDB)
    cur = con.cursor()
    cur.execute(''' SELECT item.album, item.path, album.owner FROM album, item
                        WHERE item.id = ? AND item.album == album.id
                ''', (itemid,))
    result = cur.fetchall()
    if not result:
        con.close()
        return None, None

    album = result[0][0]
    path = result[0][1].encode('utf8')
    owner = result[0][2].encode('utf8')
    print 'album=%d, path=%s, owner=%s' % (album, path, owner)
    if login != owner:
        cur.execute(''' SELECT share.sharer FROM album, share
                            WHERE album.id = share.album AND album.id = ?
                    ''', (album,))
        result = cur.fetchall()
        if (u'*',) not in result and (getunicode(login),) not in result:
            path, owner = None, None

    con.close()
    return path, owner

def album2path(path, login):
    itemid = None
    if path[:11] == '/dav/album/':
        try:
            base64 = os.path.splitext(os.path.split(path)[1])[0]
            itemid = int(base64.decode('base64').strip('abcdefghijklmnopqrs'))
            padding = '%s%d' % (''.join(PADDING[:-len(str(itemid))]), itemid)
            assert padding.encode('base64').strip() == base64
        except:
            return None, None
        path, login = getitemowner(itemid, login)
        if path is None:
            return None, None

    return path, login

def getpaths(path, login):
    paths = []
    for x in getlist(path, True):
        p = album2path(x, login)[0]
        if p:
            paths.append(isinstance(p, unicode) and p or p.decode('utf8', 'ignore'))
        else:
            paths.append(u'')
    return paths

__all__ = ['getalbumowner', 'addalbum', 'editalbum', 'delalbum',
           'additem', 'delitem', 'moveitem', 'edititem']

# vim: set sta sw=4 et:
