#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 14 Apr 2010 08:47:29 AM CST

''' Network Setting '''

import os

from system import POPEN
from system import NASError
from system import get_name_value, set_name_value
from system import lock, unlock

from service import restartservice

import pnetwork
import struct
import socket

IFDEV = 'eth0'
NETWORKCONF = '/etc/sysconfig/network'
IFCFG = '/etc/sysconfig/network-scripts/ifcfg-%s' % IFDEV

def gethostname(**kw):
    return {'hostname': os.uname()[1]}

def sethostname(hostname=None, **kw):
    if hostname is None:
        raise NASError('Bad Request')

    hostname = str(hostname).strip()
    names = {'HOSTNAME': hostname}
    set_name_value(NETWORKCONF, names)
    if os.uname()[1] != hostname:
        error, pipe = POPEN(['hostname', hostname])
        pipe.close()
        if error == 0:
            restartservice('avahi-daemon')
    return {'network': 'update hostname'}

#names = {'ipaddr': '',
#         'netmask': '',
#         'gateway': '',
#         'dhcp': '',
#         'autodns': '',
#         'dns1': '',
#         'dns2': ''
#         }
def setnetwork(dhcp=None, **kw):
    names = {}
    dns = None
    if str(dhcp) == 'no':
        names['IPADDR'] = kw.get('ipaddr', '')
        names['NETMASK'] = kw.get('netmask', '')
        names['GATEWAY'] = kw.get('gateway', '')
        _checkipaddr(**kw)
        names['BOOTPROTO'] = 'none'
        names['DEVICETYPE'] = None
        names['NOZEROCONF'] = None
        names['PEERDNS'] = 'yes'
        names['DNS1'] = kw.get('dns1', '8.8.8.8')
        names['DNS2'] = kw.get('dns2', '8.8.4.4')
        dns = {'nameserver': [names['DNS1'], names['DNS2']]}
    else:
        names['BOOTPROTO'] = 'dhcp'
        names['DEVICETYPE'] = 'avahi'
        names['NOZEROCONF'] = 'yes'
        names['IPADDR'] = None
        names['NETMASK'] = None
        names['GATEWAY'] = None
        if kw.get('autodns', 'yes') == 'no':
            names['PEERDNS'] = 'yes'
            names['DNS1'] = kw.get('dns1', '8.8.8.8')
            names['DNS2'] = kw.get('dns2', '8.8.4.4')
            dns = {'nameserver': [names['DNS1'], names['DNS2']]}
        else:
            names['DNS1'] = None
            names['DNS2'] = None
            names['PEERDNS'] = None

    change = False
    if set_name_value(IFCFG, names):
        change = True
    if dns and set_name_value('/etc/resolv.conf', dns, ['#'], ' '):
        change = True

    if not change:
        return {'network': 'set work'}

    if os.fork() > 0:
        return {'network': 'set and restart work'}

    pipe = lock('/var/tmp/network')
    (error, status) = POPEN('service network restart')
    status.close()
    unlock(pipe)

    os._exit(0)

def getnetwork(**kw):
    #network = _ipaddrshow()
    #network['gateway'] = _iprouteshow()[0]
    network = pnetwork.getnetwork()

    # hostname
    network['hostname'] = os.uname()[1]

    # nameserver
    dns = []
    names = {'nameserver': dns}
    get_name_value('/etc/resolv.conf', names, ['#'], ' ')
    if len(dns) < 1:
        network['dns1'] = ''
    else:
        network['dns1'] = dns[0]

    if len(dns) < 2:
        network['dns2'] = ''
    else:
        network['dns2'] = dns[1]

    # network setting
    names = {'BOOTPROTO': '', 'PEERDNS': 'yes', 'DNS1': None, 'DNS2': None}
    get_name_value(IFCFG, names)
    network['dhcp'] = names['BOOTPROTO'].lower() == 'dhcp' and 'yes' or 'no'
    if network['dhcp'] == 'yes':
        # dont modify resolv.conf
        if names.get('PEERDNS').lower() != 'yes':
            network['autodns'] = 'no'
        # modify resolv.conf, without DNS1 and DNS2
        elif names.get('DNS1') is None and names.get('DNS2') is None:
            network['autodns'] = 'yes'
        else:
            network['autodns'] = 'no'

    return network

NETWORK = {}
def gethwaddr(dev='eth0'):
    if NETWORK.get(dev):
        return NETWORK.get(dev)
    NETWORK[dev] = pnetwork.gethwaddr(dev)
    return NETWORK.get(dev)
    #result = _ipaddrshow(dev)
    #return result.get('hwaddr', '')

def getipaddr(dev='eth0'):
    return pnetwork.getnetwork(dev).get('ipaddr', '')
    #result = _ipaddrshow(dev)
    #return result.get('ipaddr', '')

def getvpnip(dev='tun0'):
    addrs = pnetwork._getaddr()
    for addr in addrs:
        if addr.get('name', '') == dev:
            return addr.get('local', '')
    return ''

def getdev(address):
    if not address:
        return None

    import pnetwork
    addrs = pnetwork._getaddr(False)
    links = pnetwork._getlink(False)
    for addr in addrs:
        if addr['local'] == address:
            for link in links:
                if link.get('state', 'up') == 'down':
                    continue
                if link['name'] == addr['name']:
                    return link['name'][:3]
    return None

def _checkipaddr(**kw):
    def aton(name):
        try:
            value = socket.inet_aton(kw.get(name, ''))
        except socket.error:
            raise NASError('Invalid %s %s' % (name, kw.get('name', '')))
        return struct.unpack('!I', value)[0]

    def ntoa(n):
        value = struct.pack('!I', n)
        return socket.inet_ntoa(value)

    ipaddr = aton('ipaddr')
    netmask = aton('netmask')
    gateway = aton('gateway')
    if (ipaddr & netmask) != (gateway & netmask):
        raise NASError('Invalid gateway %s' % kw.get('gateway'))
    broad = ipaddr | (~netmask & 0xffffffff)
    if broad == ipaddr or broad == gateway:
        raise NASError('%s is broadcast address' % ntoa(broad))

    return ntoa(ipaddr), ntoa(netmask), ntoa(gateway)

__all__ = ['gethostname', 'sethostname',
           'getnetwork', 'setnetwork',
           'gethwaddr', 'getipaddr']

# vim: set sta sw=4 et:
