#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Wed 14 Apr 2010 08:47:29 AM CST

# below is python-implement
# of course, we can use ip, route, ifconfig to get these values
# but, it's python way ...
import socket
import struct

def _netlink(nlmsg_type, ipv6=False):
    # struct nlmsghdr {
    #    __u32 nlmsg_len;    /* Length of message including header. */
    #    __u16 nlmsg_type;   /* Type of message content. */
    #    __u16 nlmsg_flags;  /* Additional flags. */
    #    __u32 nlmsg_seq;    /* Sequence number. */
    #    __u32 nlmsg_pid;    /* PID of the sending process. */
    # }
    # struct rtgenmsg {
    #    unsigned char rtgen_family;
    # };

    # NLM_F_REQUEST | NLM_F_ROOT |NLM_F_MATCH
    flags = 1 | 0x100 | 0x200

    if ipv6 is False:
        rtgen_family = socket.AF_INET
    elif ipv6 is True:
        rtgen_family = socket.AF_INET6
    else:
        rtgen_family = socket.AF_UNSPEC

    data = struct.pack('IHHIIB', 17, nlmsg_type, flags, 0, 0, rtgen_family)
    sock = socket.socket(socket.AF_NETLINK, socket.SOCK_DGRAM)
    sock.bind((0, 0))
    sock.send(data)

    messages = ''
    while True:
        message = sock.recv(65536)
        if len(message) < 16:
            message += sock.recv(65536)
            continue
        # message less than 16 ?
        unpack = struct.unpack('IHHII', message[:16])
        if unpack[1] == 3: # NLMSG_DONE
            break
        messages += message
    return messages

def _getroute(ipv6=None):
    messages = _netlink(26, ipv6) # RTM_GETROUTE

    gateways = []
    while messages != '':
        index = ord(messages[0])
        if index < 28:
            if index == 0:
                index = 1
            messages = messages[index:]
            continue
        if len(messages) < 28:
            break
        message = messages[:index]
        messages = messages[index:]

        # struct rtmsg {
        #     unsigned char rtm_family;   /* Address family of route */
        #     unsigned char rtm_dst_len;  /* Length of destination */
        #     unsigned char rtm_src_len;  /* Length of source */
        #     unsigned char rtm_tos;      /* TOS filter */
        #     unsigned char rtm_table;    /* Routing table ID */
        #     unsigned char rtm_protocol; /* Routing protocol; see below */
        #     unsigned char rtm_scope;    /* See below */
        #     unsigned char rtm_type;     /* See below */
        #     unsigned int  rtm_flags;
        # };
        unpack = struct.unpack('IHHII8BI', message[:28])
        family = unpack[5]
        dst_len = unpack[6]
        table = unpack[9]
        protocol = unpack[10]
        if table == 0 or table == 255: # RT_TABLE_UNSPEC, RT_TABLE_LOCAL
            continue
        if protocol == 2: # RTPROT_KERNEL
            continue
        if dst_len != 0: # default gateway
            continue
        attrs = message[28:]

        gateway = {}
        while attrs != '':
            index = ord(attrs[0])
            if index < 4:
                if index == 0:
                    index = 1
                attrs = attrs[index:]
                continue
            if len(attrs) < 4:
                break
            attr = attrs[:index]
            attrs = attrs[index:]
            rta_type = struct.unpack('HH', attr[:4])[1]
            if rta_type == 4 and len(attr) >= 8: # RTA_OIF
                gateway['index'] = struct.unpack('I', attr[4:])[0]
            elif rta_type == 5: # RTA_GATEWAY
                try: gateway['gateway'] = socket.inet_ntop(family, attr[4:])
                except ValueError: pass
            elif rta_type == 6 and len(attr) >= 8: # RTA_PRIORITY
                gateway['priority'] = struct.unpack('I', attr[4:])[0]

        if len(gateways) > 0 and gateways[0]['priority'] < gateway['priority']:
            gateways.append(gateway)
        else:
            gateways.insert(0, gateway)


    return gateways

def _getlink(ipv6=None):
    if ipv6 is True:
        ipv6 = None
    messages = _netlink(18, ipv6) # RTM_GETLINK

    links = []
    while messages != '':
        index = ord(messages[0])
        if index < 32:
            if index == 0:
                index = 1
            messages = messages[index:]
            continue
        if len(messages) < 32:
            break

        message = messages[:index]
        messages = messages[index:]

        # struct ifinfomsg {
        #     unsigned char  ifi_family; /* AF_UNSPEC */
        #     unsigned short ifi_type;   /* Device type */
        #     int            ifi_index;  /* Interface index */
        #     unsigned int   ifi_flags;  /* Device flags  */
        #     unsigned int   ifi_change; /* change mask */
        # };
        unpack = struct.unpack('IHHIIBHiII', message[:32])
        attrs = message[32:]

        link = {'name': ''}
        link['index'] = unpack[7]
        while attrs != '':
            index = ord(attrs[0])
            if index < 4:
                if index == 0:
                    index = 1
                attrs = attrs[index:]
                continue
            if len(attrs) < 4:
                break
            attr = attrs[:index]
            attrs = attrs[index:]
            rta_type = struct.unpack('HH', attr[:4])[1]
            if rta_type == 1: # ADDRESS
                ether = ['%02x' % ord(x) for x in attr[4:]]
                link['hwaddr'] = ':'.join(ether)
            # elif rta_type == 2: # BROADCAST
            elif rta_type == 3: # IFNAME
                link['name'] = attr[4:].rstrip('\0')
            elif rta_type == 4 and len(attr) >= 8: # MTU
                link['mtu'] = struct.unpack('I', attr[4:])[0]
            # elif rta_type == 6: # QDISC
            #     link['qdisc'] = attr[4:].rstrip('\0')
            elif rta_type == 16 and len(attr) >= 5:
                state = ord(attr[4])
                if state == 2:
                    link['state'] = 'down'
                elif state == 6:
                    link['state'] = 'up'
        if link['name']:
            links.append(link)
    return links

def gethwaddr(dev='eth0', state=''):
    for link in _getlink(False):
        if link['name'] == dev:
            if state and state == link.get('state'):
                return link['hwaddr']
            elif not state:
                return link['hwaddr']
    return ''

def _getaddr(ipv6=None):
    messages = _netlink(22, ipv6) # RTM_GETADDR

    addresses = []
    while messages != '':
        index = ord(messages[0])
        if index < 24:
            if index == 0:
                index = 1
            messages = messages[index:]
            continue
        if len(messages) < 24:
            break
        message = messages[:index]
        messages = messages[index:]
        # struct ifaddrmsg {
        #     unsigned char ifa_family;    /* Address type */
        #     unsigned char ifa_prefixlen; /* Prefixlength of address */
        #     unsigned char ifa_flags;     /* Address flags */
        #     unsigned char ifa_scope;     /* Address scope */
        #     int           ifa_index;     /* Interface index */
        # }
        unpack = struct.unpack('IHHII4Bi', message[:24])
        prefix = unpack[6]
        family = unpack[5]
        attrs = message[24:]
        address = {}
        address['prefix'] = prefix
        address['netmask'] = _prefix_to_mask(prefix)
        address['index'] = unpack[9]
        while attrs != '':
            index = ord(attrs[0])
            if index < 4:
                if index == 0:
                    index = 1
                attrs = attrs[index:]
                continue
            if len(attrs) < 4:
                break
            attr = attrs[:index]
            attrs = attrs[index:]
            rta_type = struct.unpack('HH', attr[:4])[1]
            if rta_type == 1:   # ADDRESS
                try: address['addr'] = socket.inet_ntop(family, attr[4:])
                except ValueError: pass
            elif rta_type == 2: # LOCAL
                try: address['local'] = socket.inet_ntop(family, attr[4:])
                except ValueError: pass
            elif rta_type == 3: # LABEL
                address['name'] = attr[4:].rstrip('\0')
            elif rta_type == 4: # BROADCAST
                try: address['broadcast'] = socket.inet_ntop(family, attr[4:])
                except ValueError:pass
            # elif rta_type == 6: # CACHEINFO
        addresses.append(address)
    return addresses

def _prefix_to_mask(bit):
    masklist = []
    maskbits = '1' * int(bit) + '0' * (32 - int(bit))
    for x in range(0, len(maskbits), 8):
        masklist.append('%d' % int(maskbits[x:x+8], 2))
    return '.'.join(masklist)

def _samenet(gateway, ipaddr, prefix):
    aton = lambda x: struct.unpack('!I', socket.inet_aton(x))[0]
    netmask = int('1' * int(prefix) + '0' * (32 - int(prefix)), 2)
    return (aton(ipaddr) & netmask) == (aton(gateway) & netmask)

def getnetwork(dev='eth0'):
    addrs = _getaddr(False)
    routes = _getroute(False)
    network = {'name': '', 'ipaddr': '', 'netmask': '', 'gateway': ''}
    if not routes:
        for addr in addrs:
            if addr['name'] == dev:
                network['name'] = addr['name']
                network['ipaddr'] = addr['local']
                network['netmask'] = _prefix_to_mask(addr['prefix'])
                network['hwaddr'] = gethwaddr(addr['name'])
                break
        return network
    route = routes[0]
    for addr in addrs:
        if addr['index'] != route['index']:
            continue
        if _samenet(route['gateway'], addr['local'], addr['prefix']):
            network['name'] = addr['name']
            network['gateway'] = route['gateway']
            network['ipaddr'] = addr['local']
            network['netmask'] = _prefix_to_mask(addr['prefix'])
            network['hwaddr'] = gethwaddr(addr['name'], 'up')
            return network
    return network

__all__ = ['gethwaddr', 'getnetwork']

# vim: set sta sw=4 et:
