#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 02:38:22 PM CST

''' SMTP Mail '''

import os
import re
import time

import socket
import ssl
import smtplib
from email.mime.text import MIMEText

from system import _all
from system import NASError
from system import get_name_value
from system import set_name_value

from system import decrypt

SMTPCONF = '/etc/nas/smtp'

def _setreceivers(receivers, emails):
    ''' convert emails to list '''
    if not isinstance(emails, list):
        emails = [emails]

    for email in emails:
        for receiver in re.split(',|;', email):
            receivers.append(receiver.strip())

def _getsmtperror(err):
    ''' convert smtpexception to dict '''
    if hasattr(err, 'smtp_code'):
        return {'code': '%s' % err.smtp_code,
                'message': err.smtp_error}
    elif hasattr(err, 'strerror'):
        return err.strerror
    else:
        return err.args[0]

def _getsenderrs(senderrs):
    ''' convert receipient response to dict '''
    messages = []
    for sender in senderrs:
        message = {}
        message['receiver'] = '%s' % sender
        message['code'] = senderrs[sender][0]
        message['message'] = senderrs[sender][1]
        messages.append({'recipient': message})
    return messages

def _notify(subject, body):
    names = _getsmtp()
    names['body'] = body
    if names['notify'] == 'yes' and names['email']:
        mails = {'body': body,
                 'subtype': 'plain',
                 'subject': subject,
                 'receiver': names['email']}
        sendemail(**mails)

    if names['mobile_notify'] == 'yes' and names['mobile_phone']:
        dest = names['mobile_phone']
        msg = '%s\n%s' % (subject, body)
        sendsms(dest=dest, msg=msg)

def sendemail(body=None, receiver=None, sender=None, **kw):
    if not _all(body, receiver):
        raise NASError('Bad Request')

    subject = kw.get('subject', '')
    subtype = kw.get('subtype', 'html')
    charset = kw.get('charset', 'utf8')

    # body = str(body)

    names = _getsmtp()
    if not names['host'] or not names['user'] or not names['pass']:
        raise NASError('No SMTP Account')

    receivers = []
    _setreceivers(receivers, receiver)

    if not sender:
        if '@' in names['user']:
            sender = names['user']
        else:
            sender = '%s@%s' % (names['user'], names['host'])

    message = MIMEText(body, subtype, charset)
    message['Date'] = time.strftime('%a, %d %b %Y %H:%M:%S %z')
    message['Subject'] = subject

    message['From'] = sender
    message['To'] = ', '.join(receivers)
    return _sendemail(names, sender, receivers, message)

def _sendemail(names, sender, receivers, message):
    try:
        if names['ssl'] == 'yes':
            server = SMTP_SSL(names['host'], int(names['port']))
        else:
            server = smtplib.SMTP(names['host'], int(names['port']))
    except (smtplib.SMTPConnectError, socket.error):
        raise NASError('Connect Error')

    # server.set_debuglevel(True)

    try:
        server.login(names['user'], names['pass'])
        senderrs = server.sendmail(sender, receivers, message.as_string())
    except smtplib.SMTPHeloError:
        raise NASError('SMTP Hello Error')
    except smtplib.SMTPAuthenticationError:
        raise NASError('SMTP Authentication Error')
    except smtplib.SMTPRecipientsRefused:
        raise NASError('SMTP Receivers Refused')
    except smtplib.SMTPHeloError:
        raise NASError('SMTP Hello Error')
    except smtplib.SMTPSenderRefused as error:
        raise NASError('SMTP Sender Refused', error.sender)
    except smtplib.SMTPDataError:
        raise NASError('SMTP Data Error')
    except smtplib.SMTPException, e:
        msg = len(e.args) > 0 and str(e.args[0]) or 'Unknown SMTP Error'
        raise NASError(msg)

    server.quit()

    for recipient in senderrs:
        receivers.remove(recipient)

    return {'sendmail': [{'success': ', '.join(receivers)},
                         {'fail': _getsenderrs(senderrs)}]}

def _getsmtp():
    ''' get smtp setting '''
    names = {'host': '',
             'port': '',
             'user': '',
             'pass': '',
             'ssl': '',
             'notify': '',
             'email': '',
             'mobile_user': '',
             'mobile_pass': '',
             'mobile_notify': '',
             'mobile_phone': '',
             }
    get_name_value(SMTPCONF, names, [])
    return names

def getsmtp(**kw):
    names = _getsmtp()
    del names['pass']
    del names['mobile_pass']
    return {'smtp': names}

import sys
import urllib
import httplib
def sendsms(dest, msg, **kw):
    host = 'api.every8d.com'
    path = '/API21/HTTP/sendSMS.ashx'
    names = {
             'mobile_user': '',
             'mobile_pass': '',
             # 'mobile_notify': '',
             # 'mobile_phone': '',
             }

    get_name_value(SMTPCONF, names, [])
    if not names['mobile_user'] or not names['mobile_pass'] or not dest:
        raise NASError('No SMS Account')

    request = {'UID': names['mobile_user'],
               'PWD': names['mobile_pass'],
               'MSG': msg,
               'DEST': dest,
               'SB': '',
               'ST': '',
               }

    path += '?%s' % urllib.urlencode(request)
    conn = httplib.HTTPConnection(host)
    conn.putrequest('GET', path)
    status = None
    headers = []
    content = ''
    try:
        conn.endheaders()
        resp = conn.getresponse()
        status = resp.status
        headers = resp.getheaders()
        content = resp.read()
        resp.close()
    except Exception as error:
        status = None
        if hasattr(error, 'strerror'):
            content = error.strerror
        else:
            content = ', '.join([str(x) for x in error.args])
    finally:
        conn.close()

    print >> sys.stderr, 'status=%s' % status
    print >> sys.stderr, 'headers=%s' % headers
    print >> sys.stderr, 'content=%s' % content

    if len(content.split(',')) == 5:
        credit, sended, cost, unsend, batch_id = content.split(',')
    else:
        raise NASError('SMS Error', content)

    return  {'mobile': {'sended': '%s' % sended,
                        'unsend': '%s' % unsend}}

def setsmtp(session=None, **kw):

    name = {}
    name['host'] = kw.get('host')
    name['user'] = kw.get('username')
    name['pass'] = kw.get('password')

    if name['pass'] is not None:
        if session is None:
            raise NASError('Bad Request')
        name['pass'] = decrypt(session, name['pass'])

    name['mobile_user'] = kw.get('mobile_user')
    name['mobile_notify'] = kw.get('mobile_notify')
    name['mobile_phone'] = kw.get('mobile_phone')
    if kw.get('mobile_pass') is not None:
        name['mobile_pass'] = decrypt(session, kw.get('mobile_pass'))

    name['ssl'] = kw.get('ssl')
    name['port'] = kw.get('port')
    name['email'] = kw.get('email')
    name['notify'] = kw.get('notify')

    if isinstance(name['port'], str) and not name['port'].isdigit():
        raise NASError('Bad Request')

    names = {}
    for item in name:
        value = name.get(item)
        if value is not None:
            names[item] = value

    mask = os.umask(077)
    set_name_value(SMTPCONF, names)
    os.umask(mask)
    return {'smtp': 'update config file'}

# Ther SMTP_SSL class in Python 2.6.2 has a bug, this is the fixed version
class SMTP_SSL(smtplib.SMTP):
    """ This is a subclass derived from SMTP that connects over an SSL encrypted
    socket (to use this class you need a socket module that was compiled with SSL
    support). If host is not specified, '' (the local host) is used. If port is
    omitted, the standard SMTP-over-SSL port (465) is used. keyfile and certfile
    are also optional - they can contain a PEM formatted private key and
    certificate chain file for the SSL connection.
    """
    def __init__(self, host='', port=0, local_hostname=None,
                 keyfile=None, certfile=None,
                 timeout=socket._GLOBAL_DEFAULT_TIMEOUT):
        self.keyfile = keyfile
        self.certfile = certfile
        smtplib.SMTP.__init__(self, host, port, local_hostname, timeout)
        self.default_port = smtplib.SMTP_SSL_PORT

    def _get_socket(self, host, port, timeout):
        if self.debuglevel > 0: print>>stderr, 'connect:', (host, port)
        new_socket = socket.create_connection((host, port), timeout)
        new_socket = ssl.wrap_socket(new_socket, self.keyfile, self.certfile)
        self.file = smtplib.SSLFakeFile(new_socket)
        return new_socket

# vim: set sta sw=4 et:
