#!/usr/bin/env python
#fileencoding: utf-8
#Author: Liu DongMiao <liudongmiao@gmail.com>
#Created  : Thu 06 May 2010 09:35:55 AM CST

# useradd, userdel

import os
import pwd
import time
import fcntl

from system import WEBUSER
from system import NASError
from system import _all
from system import POPEN
from system import ADMIN
from system import getgid
from system import _mkdir
from system import decrypt
from system import backup, unbackup

from cStringIO import StringIO

import hashlib

HOMEDIR = '/home'
REALM = 'Network Attached Storage'

def _getusers():
    users = []
    for pw in pwd.getpwall():
        if pw.pw_dir.startswith(HOMEDIR):
            users.append(pw.pw_name)
    return users

def getusers(**kw):
    ''' Get Users
        return: name, comment
    '''
    users = []
    for pw in pwd.getpwall():
        if pw.pw_dir.startswith(HOMEDIR):
            _checkhomedir(pw)
            users.append({'name': pw.pw_name, 'comment': pw.pw_gecos})
    return {'user': users}

def _checkhomedir(pw):
    if not hasattr(pw, 'pw_dir'):
        try:
            pw = pwd.getpwnam(pw)
        except KeyError:
            raise NASError('User Not Exists', pw)
    _mkdir(pw.pw_dir)

    try:
        os.chmod(pw.pw_dir, 0754)
    except os.error:
        pass
    gid = getgid(WEBUSER)
    try:
        os.chown(pw.pw_dir, pw.pw_uid, gid)
    except os.error:
        pass
    return pw.pw_dir

def _adduser(username, password, comment=None):
    for pw in pwd.getpwall():
        if pw.pw_name == username:
            raise NASError('User Exists', username)

    backup('/etc/passwd')
    command = ['useradd', '-g', 'nas', '-m', username]
    if isinstance(comment, str):
        command.append('-c')
        command.append(comment)
    (error, unix) = POPEN(command)
    result = unix.read()
    unix.close()
    if error != 0:
        raise NASError('System Error', result)

    unbackup('/etc/passwd')
    _checkhomedir(pwd.getpwnam(username))
    _changepass(username, password)
    return getusers()

def adduser(username=None, password=None, session=None, comment=None, **kw):
    ''' Add a new user
        param : username
              : password
              : hash -> for decrpt password
              : comment  -> optional
        return: (False, reason), (True, None)
    '''
    if not _all(username, password, session):
        raise NASError('Bad Request')

    username = str(username)
    password = str(password)
    session = str(session)

    password = decrypt(session, password)
    return _adduser(username, password, comment)

def deluser(username=None, **kw):
    if username is None:
        raise NASError('Bad Request')

    username = str(username)
    try:
        pw = pwd.getpwnam(username)
    except KeyError:
        raise NASError('User Not Exists', username)

    if pw.pw_dir[:5] != HOMEDIR or pw.pw_shell != '/bin/bash':
        raise NASError('Invalid User', username)

    if username == 'admin':
        raise NASError('Invalid User', username)

    backup('/etc/passwd')
    (error, unix) = POPEN(['userdel', username])
    result = unix.read()
    unix.close()
    unbackup('/etc/passwd')
    if error != 0:
        raise NASError('System Error', result)

    def del_record(name, mode):
        content = StringIO()
        found = False
        _file = open(name, 'r')
        while True:
            line = _file.readline()
            if not line:
                break
            if line.startswith('#'):
                continue
            if line.startswith('%s:' % username):
                found = True
            else:
                content.write(line)
        _file.close()

        data = content.getvalue()
        content.close()

        if found:
            backup(name)
            FLAG = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
            fd = os.open('%s.tmp' % name, FLAG, mode)
            fcntl.lockf(fd, fcntl.LOCK_EX)
            ret = os.write(fd, data)
            os.fsync(fd)
            fcntl.lockf(fd, fcntl.LOCK_UN)
            os.close(fd)
            if ret != len(data):
                raise NASError('write data error', name)
            os.rename('%s.tmp' % name, name)
            unbackup(name)

    del_record('/etc/nas/smbpasswd', 0600)
    del_record('/etc/nas/htdigest', 0644)

    return getusers()

def _checkpass(username, password):
    try:
        pw = pwd.getpwnam(username)
    except KeyError:
        return False

    if pw.pw_dir[:5] != HOMEDIR or pw.pw_shell != '/bin/bash':
        return False

    digest = hashlib.md5('%s:%s:%s' % (username, REALM, password)).hexdigest()
    line = '%s:%s:%s' % (username, REALM, digest)

    retcode = False
    try:
        rfile = open('/etc/nas/htdigest', 'r')
    except IOError:
        return False

    for x in rfile:
        if x.strip() == line:
            _checkhomedir(pw)
            retcode = True
            break

    return retcode

    # try:
    #     salt = spwd.getspnam(username).sp_pwd
    # except KeyError:
    #     return False

    # if crypt.crypt(password, salt) == salt:
    #     _checkhomedir(pw)
    #     return True

def _changepass(username, password):
    if not _checkpass(username, password):
        backup('/etc/nas/smbpasswd')
        backup('/etc/nas/htdigest')

        _change_samba(username, password)
        _change_htdigest(username, password)

        unbackup('/etc/nas/smbpasswd')
        unbackup('/etc/nas/htdigest')

    backup('/etc/passwd')
    backup('/etc/shadow')
    (error, job) = POPEN('echo %s:%s' % (username, password))
    (error, pipe) = POPEN('chpasswd', job)
    job.close()
    pipe.close()
    unbackup('/etc/shadow')
    unbackup('/etc/passwd')

def _change_samba(username, password):

    date = int(time.time())
    if date < 1272643200: # 2010/05/01 00:00:00 CST
        date = 1272643200

    changed = False
    # Samba use md4
    md4 = hashlib.new('md4')
    md4.update(password.encode('utf-16le'))
    nthash = md4.hexdigest().upper()
    passwd = pwd.getpwnam(username)
    pw_name = passwd.pw_name
    pw_uid = passwd.pw_uid
    lhash = 'X' * 32
    flags = '[U          ]'
    lct = 'LCT-%s' % hex(date)[2:].upper()
    content = StringIO()
    if os.path.exists('/etc/nas/smbpasswd'):
        found = False
        _file = open('/etc/nas/smbpasswd', 'r')
        while True:
            line = _file.readline()
            if not line:
                break
            if line.startswith('#'):
                continue
            # user:uid:lhash:nthash:flags:lct
            try:
                (user, uid, olhash, onthash, oflags, olct) = line.split(':')
            except ValueError:
                continue
            if user == pw_name and (uid != str(pw_uid)
                    or onthash.upper() != nthash):
                found = True
                changed = True
                content.write('%s:%s:%s:%s:%s:%s'
                        % (pw_name, pw_uid, lhash, nthash, flags, lct))
                content.write(os.linesep)
            else:
                content.write(line)
        if not found:
            changed = True
            content.write('%s:%s:%s:%s:%s:%s'
                    % (pw_name, pw_uid, lhash, nthash, flags, lct))
            content.write(os.linesep)
        _file.close()
    else:
        changed = True
        content.write('%s:%s:%s:%s:%s:%s'
                % (pw_name, pw_uid, lhash, nthash, flags, lct))
        content.write(os.linesep)

    data = content.getvalue()
    content.close()
    if changed:
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
        pipe = os.open('/etc/nas/smbpasswd.tmp', flags, 0600)
        fcntl.lockf(pipe, fcntl.LOCK_EX)
        ret = os.write(pipe, data)
        os.fsync(pipe)
        fcntl.lockf(pipe, fcntl.LOCK_UN)
        os.close(pipe)
        if ret != len(data):
            raise NASError('write data error', '/etc/nas/smbpasswd')
        os.rename('/etc/nas/smbpasswd.tmp', '/etc/nas/smbpasswd')

def _change_htdigest(username, password):
    # htdigest

    changed = False
    content = StringIO()
    digest = hashlib.md5('%s:%s:%s' % (username, REALM, password)).hexdigest()
    if os.path.exists('/etc/nas/htdigest'):
        found = False
        _file = open('/etc/nas/htdigest', 'r')
        while True:
            line = _file.readline()
            if not line:
                break
            if line.startswith('#'):
                continue
            # user:realm:hash
            try:
                (user, realm, odigest) = line.split(':')
            except ValueError:
                continue
            if user == username and (realm != REALM
                    or odigest != digest):
                found = True
                changed = True
                content.write('%s:%s:%s' % (username, REALM, digest))
                content.write(os.linesep)
            else:
                content.write(line)
        if not found:
            changed = True
            content.write('%s:%s:%s' % (username, REALM, digest))
            content.write(os.linesep)
        _file.close()
    else:
        changed = True
        content.write('%s:%s:%s' % (username, REALM, digest))
        content.write(os.linesep)
    data = content.getvalue()
    content.close()
    if changed:
        flags = os.O_WRONLY | os.O_CREAT | os.O_TRUNC | os.O_SYNC
        pipe = os.open('/etc/nas/htdigest.tmp', flags, 0644)
        fcntl.lockf(pipe, fcntl.LOCK_EX)
        ret = os.write(pipe, data)
        os.fsync(pipe)
        fcntl.lockf(pipe, fcntl.LOCK_UN)
        os.close(pipe)
        if ret != len(data):
            raise NASError('write data error', '/etc/nas/htdigest')
        os.rename('/etc/nas/htdigest.tmp', '/etc/nas/htdigest')

    return True

def edituser(username=None, password=None, newpassword=None,
        session=None, login=None, **kw):

    # admin is allowed to change password without [old]password
    if not _all(username, newpassword):
        raise NASError('Bad Request')

    if login is None:
        raise NASError('Not Login')

    username = str(username)
    newpassword = str(newpassword)
    session = str(session)

    if login != ADMIN or ADMIN == username:
        if password is None:
            raise NASError('Bad Request')
        password = decrypt(session, str(password))
        if not _checkpass(username, password):
            raise NASError('Wrong Username or Password')
    newpassword = decrypt(session, newpassword)
    _changepass(username, newpassword)
    try: os.chmod('/etc/nas/htdigest', 0644)
    except: pass
    return {'edituser': 'update password'}

__all__ = ['getusers', 'adduser', 'deluser', 'edituser']

# vim: set sta sw=4 et:
