﻿function Webdav(option) {
    if(!option)return;
    this.url =option.url;
    this.request = null;
    this.asynchronous = true;
    this.username = option.username||'';
    this.password = option.password||'';
}

Webdav.prototype.setSynchronous = function() {
    this.asynchronous = false;
}

Webdav.prototype.isAsynchronous = function() {
    return this.asynchronous;
}

Webdav.prototype.setAsynchronous = function () {
    this.asynchronous = true;
}

Webdav.prototype.getRequest = function () {
    try {
         return new XMLHttpRequest();
    }
    catch (e) {
         try {
              return new ActiveXObject("Msxml2.XMLHTTP");
          }
         catch (e) {
              return new ActiveXObject("Microsoft.XMLHTTP");
          }
    }
    
    alert ("Browser does not support HTTP Request");
    return null;
}
Webdav.prototype.openRequest = function(handler, method, path, additionalHeaders) {
    var request = this.getRequest();
    if (request === null) {
        return null;
    }
    request.onreadystatechange = this.wrapperHandler(handler, request);
    var url = this.url + path;
    request.open(method, url, this.asynchronous);
    if (this.username) {
        var auth = make_base_auth(this.username, this.password);
		var auth = "Session " + window.top.SessionID;
		request.setRequestHeader('Authorization', auth);
    }
    if ( additionalHeaders ) {
        this.setAdditionalHeaders(request, additionalHeaders);
    }
    request.setRequestHeader('Session', window.top.SessionID);
    return request;
}
Webdav.prototype.wrapperHandler = function(handler, request) {
    var self = this;
    function Handler() {
        handler = {
            onSuccess: handler.onSuccess || function(){},
            onError: handler.onError || function(){},
            onComplete: handler.onComplete || function(){}
        }
        this.execute = function() {
            if (request.readyState == 4) {
                var result = self.parseResponse(request);
                request = null;
                if ( self.isSuccess(result.status) ) {
                    handler.onSuccess.call('', result);
                } else {
                    handler.onError.call('', result);
                }
                
                handler.onComplete.call('', result);
            };
        };
    };
    return (new Handler().execute);
};
Webdav.prototype.parseResponse = function(request){
    // define source of response content
    if ( request.getResponseHeader("content-type") &&
            request.getResponseHeader("content-type").indexOf("xml") >= 0 )
    {
        var content = request.responseXML;                    
    } else {
        var content = request.responseText;                    
    }
    
    var result = {
        status: request.status,
        statusstring: this.STATUS_CODES[request.status.toString()],
        headers: this.parseHeaders(request.getAllResponseHeaders()),
        content: content
    };
    
    return result; 
}
Webdav.prototype.parseHeaders = function(headers) {
    var headerPattern = /([^:]+):.(.+)/;
    var lines = headers.split('\n');
    var hash = {}; 
    var name = ''; 
    var value = '';
    for (var i = 0; i < lines.length; i++) {
        var header = lines[i].match(headerPattern);
        if ( header != null && 
                typeof(header[1]) != 'undefined') 
        {
            name = header[1];
            if (typeof(header[2]) != 'undefined') {
                value = header[2];
            } else {
                value = '';
            }
            hash[name] = value;
        }
    };
    
    return hash;
};
Webdav.prototype.setAdditionalHeaders = function (request, additionalHeaders) {
    for( var headerName in additionalHeaders ) {
        if (!additionalHeaders.hasOwnProperty(headerName)) continue;
        var headerValue = additionalHeaders[headerName];
        request.setRequestHeader(headerName, headerValue);
    }
}
Webdav.prototype.isSuccess = function(status){
    return (status >= 200 && status < 300 ) ||
                  status == 304;
}
Webdav.prototype.STATUS_CODES = {
    '100': 'Continue',
    '101': 'Switching Protocols',
    '102': 'Processing',
    '200': 'OK',
    '201': 'Created',
    '202': 'Accepted',
    '203': 'None-Authoritive Information',
    '204': 'No Content',
    '1223': 'No Content',
    '205': 'Reset Content',
    '206': 'Partial Content',
    '207': 'Multi-Status',
    '300': 'Multiple Choices',
    '301': 'Moved Permanently',
    '302': 'Found',
    '303': 'See Other',
    '304': 'Not Modified',
    '305': 'Use Proxy',
    '307': 'Redirect',
    '400': 'Bad Request',
    '401': 'Unauthorized',
    '402': 'Payment Required',
    '403': 'Forbidden',
    '404': 'Not Found',
    '405': 'Method Not Allowed',
    '406': 'Not Acceptable',
    '407': 'Proxy Authentication Required',
    '408': 'Request Time-out',
    '409': 'Conflict',
    '410': 'Gone',
    '411': 'Length Required',
    '412': 'Precondition Failed',
    '413': 'Request Entity Too Large',
    '414': 'Request-URI Too Large',
    '415': 'Unsupported Media Type',
    '416': 'Requested range not satisfiable',
    '417': 'Expectation Failed',
    '422': 'Unprocessable Entity',
    '423': 'Locked',
    '424': 'Failed Dependency',
    '500': 'Internal Server Error',
    '501': 'Not Implemented',
    '502': 'Bad Gateway',
    '503': 'Service Unavailable',
    '504': 'Gateway Time-out',
    '505': 'HTTP Version not supported',
    '507': 'Insufficient Storage'
};
function make_base_auth(user, password) {
  var tok = user + ':' + password;
  var hash = Base64.encode(tok);
  return "Basic " + hash;
}
var Base64 = {
    _keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",
    encode : function (input) {
        var output = "";
        var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
        var i = 0;

        input = Base64._utf8_encode(input);

        while (i < input.length) {

            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }

            output = output +
            this._keyStr.charAt(enc1) + this._keyStr.charAt(enc2) +
            this._keyStr.charAt(enc3) + this._keyStr.charAt(enc4);

        }

        return output;
    },
    decode : function (input) {
        var output = "";
        var chr1, chr2, chr3;
        var enc1, enc2, enc3, enc4;
        var i = 0;
        input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
        while (i < input.length) {
            enc1 = this._keyStr.indexOf(input.charAt(i++));
            enc2 = this._keyStr.indexOf(input.charAt(i++));
            enc3 = this._keyStr.indexOf(input.charAt(i++));
            enc4 = this._keyStr.indexOf(input.charAt(i++));
            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            output = output + String.fromCharCode(chr1);
            if (enc3 != 64) {
                output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
                output = output + String.fromCharCode(chr3);
            }
        }
        output = Base64._utf8_decode(output);
        return output;

    },
    _utf8_encode : function (string) {
        string = string.replace(/\r\n/g,"\n");
        var utftext = "";

        for (var n = 0; n < string.length; n++) {

            var c = string.charCodeAt(n);

            if (c < 128) {
                utftext += String.fromCharCode(c);
            }
            else if((c > 127) && (c < 2048)) {
                utftext += String.fromCharCode((c >> 6) | 192);
                utftext += String.fromCharCode((c & 63) | 128);
            }
            else {
                utftext += String.fromCharCode((c >> 12) | 224);
                utftext += String.fromCharCode(((c >> 6) & 63) | 128);
                utftext += String.fromCharCode((c & 63) | 128);
            }

        }

        return utftext;
    },
    _utf8_decode : function (utftext) {
        var string = "";
        var i = 0;
        var c = c1 = c2 = 0;
        while ( i < utftext.length ) {
            c = utftext.charCodeAt(i);
            if (c < 128) {
                string += String.fromCharCode(c);
                i++;
            }
            else if((c > 191) && (c < 224)) {
                c2 = utftext.charCodeAt(i+1);
                string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
                i += 2;
            }
            else {
                c2 = utftext.charCodeAt(i+1);
                c3 = utftext.charCodeAt(i+2);
                string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
                i += 3;
            }
        }
        return string;
    }
}
Webdav.prototype.ExtensionMethod = function (handler, path, options) {
    if (typeof(options) == 'undefined' || ! options.method ) return;
  if (typeof(options.body) == 'undefined') options.body = null;
  
    var options = {
            method: options.method,
            headers: options.headers || {},
            body: options.body
        };
    
    var request = this.openRequest(handler, options.method, path, options.headers);
    request.send( options.body );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.GET = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = { 
        additional_headers: options.additional_headers || {}
    };
    
    var request = this.openRequest(handler, 'GET', path || '', options.additional_headers);
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.PUT = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            content: options.content || '',
            content_type: options.content_type || '',			
            content_range: options.content_range || '',
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || ''                         
        };

    var request = this.openRequest(handler, 'PUT', path.replace(this.url,'') || '', options.additional_headers);
    if (options.content_type) {
      request.setRequestHeader("Content-Type", options.content_type);
    } 
	if (options.content_range) {
      request.setRequestHeader("Content-Range", options.content_range);
    } 
    if (options.locktoken) {
        request.setRequestHeader('If', '(<' + options.locktoken + '>)');
    };
    //if(document.all){
	   request.send(options.content);
	//}
	//else {
		//request.sendAsBinary(options.content);
	//}

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};

Webdav.prototype.DELETE = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};
    var options = { 
        locktoken: options.locktoken || '',
        additional_headers: options.additional_headers || {}
    };
	//for(var i=0;i<path.length;i++){
		var request = this.openRequest(handler, 'DELETE', path.replace(this.url,'') || '', options.additional_headers);
		if (options.locktoken) {
			request.setRequestHeader('If', '(<' + options.locktoken + '>)');
		};
		request.send(null);
	//}
	if ( ! this.isAsynchronous() ) {    
		var result = this.parseResponse(request);
		request = null;
		return result;
	}
};
Webdav.prototype.OPTIONS = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = { 
        additional_headers: options.additional_headers || {}
    };

    var request = this.openRequest(handler, 'OPTIONS', path || '', options.additional_headers);
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.MKCOL = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};
    
    var options = { 
        locktoken: options.locktoken || '',
        additional_headers: options.additional_headers || {}
    };


    var request = this.openRequest(handler, 'MKCOL', path.replace(this.url,'') || '', options.additional_headers);
    if (options.locktoken) {
        request.setRequestHeader('If', '(<' + options.locktoken + '>)');
    };
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    } 
}
Webdav.prototype.COPY = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};
    var options = {
            depth: options.depth,
            destination: options.destination || '',
            overwrite: ! options.hasOwnProperty('overwrite') || options.overwrite,
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };

		var request = this.openRequest(handler, 'COPY', path.replace(this.url,'') || '', options.additional_headers);
		if ( typeof(options.depth) != 'undefined' ) request.setRequestHeader('Depth', options.depth);
		request.setRequestHeader('Destination', options.destination);
		if (options.overwrite) {
			request.setRequestHeader('Overwrite', 'T');
		} else {
			request.setRequestHeader('Overwrite', 'F');        
		};
		if (options.locktoken) {
			request.setRequestHeader('If', '<' + options.locktoken + '>');
		};
		request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.MOVE = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};    
    var options = {
            depth: options.depth,
            destination: options.destination || '',
            overwrite: ! options.hasOwnProperty('overwrite') || options.overwrite,
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };		
		
    var request = this.openRequest(handler, 'MOVE', path.replace(this.url,'') || '', options.additional_headers);
    if ( typeof(options.depth) != 'undefined' ) request.setRequestHeader('Depth', options.depth);
    request.setRequestHeader('Destination', options.destination);
    if (options.overwrite) {
        request.setRequestHeader('Overwrite', 'T');
    } else {
        request.setRequestHeader('Overwrite', 'F');        
    };
    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.HEAD = function(handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = { 
        additional_headers: options.additional_headers || {}
    };
    
    var request = this.openRequest(handler, 'HEAD', path || '', options.additional_headers);
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.PROPFIND = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            depth: options.depth,
            operation: options.operation || 'allprop',
            properties_list: options.properties_list || new Array(),
            additional_headers: options.additional_headers || {}            
        };
    var request = this.openRequest(handler, 'PROPFIND', path, options.additional_headers);
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
    switch(options.operation) {
    case 'propname':
        xml += '<D:propfind xmlns:D="DAV:">\n';
        xml += '    <propname/>\n';
        xml += '</propfind>\n';
        break;
    case 'properties_list':
        xml += '<D:propfind xmlns:D="DAV:">\n';
        xml += '    <D:prop>\n';
        for (var i = 0; i < options.properties_list.length; i++) {
            xml += '<D:' + options.properties_list[i].replace(/\s/g, "") + '/>\n';
        }
        xml += '    </D:prop>\n';
        xml += '</D:propfind>\n';
        break;
    case 'allprop':
    default:
        xml += '<D:propfind xmlns:D="DAV:">\n';
        xml += '    <D:allprop/>\n';
        xml += '</D:propfind>\n';
        break;
    }
    if ( typeof(options.depth) != 'undefined' ) request.setRequestHeader('Depth', options.depth);
    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');
    request.send( xml );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.PROPPATCH = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            set_properties_list: options.set_properties_list || {},
            remove_properties_list: options.remove_properties_list || new Array(),
            locktoken: options.locktoken || '',            
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'PROPPATCH', path, options.additional_headers);
    
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
    xml += '<D:propertyupdate xmlns:D="DAV:">\n';
    xml += '    <D:set>\n';
    xml += '        <D:prop>\n';
    for( var propName in options.set_properties_list ) {
        if (!options.set_properties_list.hasOwnProperty(propName)) continue;
        var propValue = options.set_properties_list[propName];
        xml += '            <D:' + propName.replace(/\s/g, "") + '>\n';
        xml += '                ' + propValue + '\n';
        xml += '            </D:' + propName.replace(/\s/g, "") + '>\n';
    }
    xml += '        </D:prop>\n';
    xml += '    </D:set>\n';
    xml += '    <D:remove>\n';
    xml += '        <D:prop>\n';
    for (var i = 0; i < options.remove_properties_list.length; i++) {
        xml += '<D:' + options.remove_properties_list[i].replace(/\s/g, "") + '/>\n';
    }
    xml += '        </D:prop>\n';
    xml += '    </D:remove>\n';            

    xml += '</D:propertyupdate>\n';

    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    request.send( xml );
    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.LOCK = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            owner: options.owner || '',
            scope: options.scope || 'exclusive',
            depth: options.depth,
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };

    if (!options.timeout) {
        options.timeout = "Infinite, Second-60480000";
    } else {
        options.timeout = 'Second-' + options.timeout;
    }

    var request = this.openRequest(handler, 'LOCK', path, options.additional_headers);

    var xml = '<?xml version="1.0" encoding="UTF-8"?>\n'
        + '<D:lockinfo xmlns:D="DAV:">\n'
        + '    <D:lockscope><D:' + options.scope + ' /></D:lockscope>\n'
        + '    <D:locktype><D:write/></D:locktype>\n'
        + '    <D:owner>\n'
        + '        <D:href>\n' 
        +           encodeURI(options.owner) + '\n' 
        + '        </D:href>\n'
        + '    </D:owner>\n'
        + '</D:lockinfo>\n';
    
    if ( typeof(options.depth) != 'undefined' ) request.setRequestHeader('Depth', options.depth);
    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');
    request.setRequestHeader('Timeout', options.timeout);

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };

    request.send( xml );
    
    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
};
Webdav.prototype.UNLOCK = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};
    
    var options = { 
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}
    };

    var request = this.openRequest(handler, 'UNLOCK', path, options.additional_headers);

    request.setRequestHeader("Lock-Token", '<' + options.locktoken + '>');
    request.send(null);
    
    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.VERSIONCONTROL = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            version_href_list: options.version_href_list || [],
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'VERSION-CONTROL', path, options.additional_headers);
    
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
    xml += '<D:version-control xmlns:D="DAV:">\n';
    for (var i = 0; i < options.version_href_list.length; i++) {
        xml += '    <D:version>\n';
        xml += '        <D:href>\n';
        xml += '            ' + options.version_href_list[i] + '\n';
        xml += '        </D:href>\n';
        xml += '    </D:version>\n';
    }    
    xml += '</D:version-control>\n';

    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    
    request.send( xml );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.CHECKOUT = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'CHECKOUT', path, options.additional_headers);

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}

Webdav.prototype.CHECKIN = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'CHECKIN', path, options.additional_headers);

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}

Webdav.prototype.UNCHECKOUT = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            locktoken: options.locktoken || '',
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'UNCHECKOUT', path, options.additional_headers);

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };
    
    request.send(null);

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}

Webdav.prototype.REPORT = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            depth: options.depth,
            type: options.type || 'version-tree',
            properties_list: options.properties_list || new Array(),
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'REPORT', path, options.additional_headers);
    
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
    switch(options.type) {
    case 'version-tree':
    default:
        xml += '<D:version-tree xmlns:D="DAV:">\n';
        xml += '    <D:prop>\n';
        for (var i = 0; i < options.properties_list.length; i++) {
            xml += '<D:' + options.properties_list[i].replace(/\s/g, "") + '/>\n';
        }
        xml += '    </D:prop>\n';
        xml += '</D:version-tree>\n';
        break;
    }

    if ( typeof(options.depth) != 'undefined' ) request.setRequestHeader('Depth', options.depth);
    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');
    request.send( xml );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}


Webdav.prototype.ORDERPATCH = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            ordering_type_href: options.ordering_type_href || '',
            order_members: options.order_members || new Array(),
             locktoken:  options.locktoken || '',            
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'ORDERPATCH', path, options.additional_headers);
    
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
    xml += '<D:orderpatch xmlns:D="DAV:">\n';
    if ( options.ordering_type_href ) {
        xml += '    <D:ordering-type>\n';
        xml += '        <D:href>' + encodeURI(options.ordering_type_href) + '</D:href>\n';
          xml += '    </D:ordering-type>\n';
    }
    for (var i = 0; i < options.order_members.length; i++ ) {
        xml += '    <D:order-member>\n';
        xml += '        <D:segment>' + encodeURI(options.order_members[i]['segment']) + '</D:segment>\n';    
        switch(options.order_members[i]['position']) {
        case 'first':
        case 'last':        
            xml += '        <D:position>' + options.order_members[i]['position'] + '</D:position>\n';
            break;
        case 'before':
        case 'after':        
            xml += '        <D:position>\n';
            xml += '            <D:' + options.order_members[i]['position'] + '>\n';
            xml += '                <D:segment>' + encodeURI(options.order_members[i]['parent_segment']) + '</D:segment>\n'; 
            xml += '            </D:' + options.order_members[i]['position'] + '>\n';
            xml += '        </D:position>\n';
            break;
        }
        xml += '    </D:order-member>\n';
    }
    xml += '</D:orderpatch>\n';
        
    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');

    if (options.locktoken) {
        request.setRequestHeader('If', '<' + options.locktoken + '>');
    };

    request.send( xml );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}
Webdav.prototype.SEARCH = function (handler, path, options) {
    if (typeof(options) == 'undefined') options = {};

    var options = {
            search_language: options.search_language || 'xpath',
            search_request: options.search_request || '',
            additional_headers: options.additional_headers || {}            
        };
    
    var request = this.openRequest(handler, 'SEARCH', path, options.additional_headers);
    
    var xml = '<?xml version="1.0" encoding="UTF-8" ?>\n';
        xml += '<D:searchrequest xmlns:D="DAV:">\n';

    switch(options.search_language) {
        case 'sql':
            xml += '    <D:sql>\n';
               xml += '        ' + options.search_request + '\n';
            xml += '    </D:sql>\n';
            break;
        case 'xpath':
        default:
            xml += '    <D:xpath>\n';
               xml += '        ' + options.search_request + '\n';
            xml += '    </D:xpath>\n';
            break;
    }
    xml += '</D:searchrequest>\n';

    request.setRequestHeader('Content-Type', 'text/xml; charset=UTF-8');
    request.send( xml );

    if ( ! this.isAsynchronous() ) {    
        var result = this.parseResponse(request);
        request = null;
        return result;
    }
}